==> crypto-crawler-2.1.8.crate: crypto-crawler-2.1.8/tests/utils/mod.rs <==
#[allow(unused_macros)]
macro_rules! gen_test_code {
    ($crawl_func:ident, $exchange:expr, $market_type:expr, $symbol:expr, $msg_type:expr) => {{
        thread_local! {
            static MESSAGES: RefCell<Vec<Message>> = RefCell::new(Vec::new());
        }

        let on_msg = Arc::new(Mutex::new(|msg: Message| {
            MESSAGES.with(|messages| messages.borrow_mut().push(msg))
        }));
        let symbols = vec![$symbol.to_string()];
        $crawl_func($exchange, $market_type, Some(&symbols), on_msg, Some(0));

        MESSAGES.with(|slf| {
            let messages = slf.borrow();

            assert!(!messages.is_empty());
            assert_eq!(messages[0].exchange, $exchange.to_string());
            assert_eq!(messages[0].market_type, $market_type);
            assert_eq!(messages[0].msg_type, $msg_type);
        });
    }};
}

#[allow(unused_macros)]
macro_rules! gen_test_snapshot_code {
    ($crawl_func:ident, $exchange:expr, $market_type:expr, $symbol:expr, $msg_type:expr) => {{
        thread_local! {
            static MESSAGES: RefCell<Vec<Message>> = RefCell::new(Vec::new());
        }

        let on_msg = Arc::new(Mutex::new(|msg: Message| {
            MESSAGES.with(|messages| messages.borrow_mut().push(msg))
        }));
        let symbols = vec![$symbol.to_string()];
        $crawl_func($exchange, $market_type, Some(&symbols), on_msg, Some(0));

        MESSAGES.with(|slf| {
            let messages = slf.borrow();

            assert!(!messages.is_empty());
            assert_eq!(messages[0].exchange, $exchange.to_string());
            assert_eq!(messages[0].market_type, $market_type);
            assert_eq!(messages[0].msg_type, $msg_type);
        });
    }};
}

#[allow(unused_macros)]
macro_rules! gen_test_snapshot_without_symbol_code {
    ($crawl_func:ident, $exchange:expr, $market_type:expr, $msg_type:expr) => {{
        let messages: Arc<Mutex<Vec<Message>>> = Arc::new(Mutex::new(Vec::new()));
        let messages_clone = messages.clone();
        let on_msg = Arc::new(Mutex::new(move |msg: Message| {
            messages_clone.lock().unwrap().push(msg)
        }));
        let symbols = if $market_type == MarketType::Spot {
            let spot_symbols = fetch_symbols_retry($exchange, $market_type);
            get_hot_spot_symbols($exchange, &spot_symbols)
        } else {
            fetch_symbols_retry($exchange, $market_type)
        };
        $crawl_func($exchange, $market_type, Some(&symbols), on_msg, Some(0));

        let messages = messages.lock().unwrap();
        assert!(!messages.is_empty());
        assert_eq!(messages.len(), symbols.len());

        assert_eq!(messages[0].exchange, $exchange.to_string());
        assert_eq!(messages[0].market_type, $market_type);
        assert_eq!(messages[0].msg_type, $msg_type);
    }};
}

#[allow(unused_macros)]
macro_rules! gen_test_crawl_candlestick {
    ($exchange:expr, $market_type:expr) => {{
        let messages: Arc<Mutex<Vec<Message>>> = Arc::new(Mutex::new(Vec::new()));
        let messages_clone = messages.clone();
        let on_msg = Arc::new(Mutex::new(move |msg: Message| {
            messages_clone.lock().unwrap().push(msg)
        }));
        crawl_candlestick($exchange, $market_type, None, on_msg, Some(0));
        let messages = messages.lock().unwrap();
        assert!(!messages.is_empty());
        assert_eq!(messages[0].exchange, EXCHANGE_NAME.to_string());
        assert_eq!(messages[0].market_type, $market_type);
        assert_eq!(messages[0].msg_type, MessageType::Candlestick);
    }};
}

==> crypto-crawler-3.0.3.crate: crypto-crawler-3.0.3/tests/utils/mod.rs <==
#[allow(unused_macros)]
macro_rules! test_one_symbol {
    ($crawl_func:ident, $exchange:expr, $market_type:expr, $symbol:expr, $msg_type:expr) => {{
        let (tx, rx) = std::sync::mpsc::channel();
        let mut messages = Vec::new();
        let symbols = vec![$symbol.to_string()];
        $crawl_func($exchange, $market_type, Some(&symbols), tx, Some(0));

        for msg in rx {
            messages.push(msg);
        }

        assert!(!messages.is_empty());
        assert_eq!(messages[0].exchange, $exchange.to_string());
        assert_eq!(messages[0].market_type, $market_type);
        assert_eq!(messages[0].msg_type, $msg_type);
    }};
}

#[allow(unused_macros)]
macro_rules! test_all_symbols {
    ($crawl_func:ident, $exchange:expr, $market_type:expr, $msg_type:expr) => {{
        let (tx, rx) = std::sync::mpsc::channel();
        let mut messages = Vec::new();
        let symbols = if $market_type == MarketType::Spot {
            let spot_symbols = fetch_symbols_retry($exchange, $market_type);
            get_hot_spot_symbols($exchange, &spot_symbols)
        } else {
            fetch_symbols_retry($exchange, $market_type)
        };
        $crawl_func($exchange, $market_type, Some(&symbols), tx, Some(0));

        for msg in rx {
            messages.push(msg);
        }

        assert!(!messages.is_empty());
        assert_eq!(messages[0].exchange, $exchange.to_string());
        assert_eq!(messages[0].market_type, $market_type);
        assert_eq!(messages[0].msg_type, $msg_type);
    }};
}

#[allow(unused_macros)]
macro_rules! gen_test_crawl_candlestick {
    ($exchange:expr, $market_type:expr) => {{
        let (tx, rx) = std::sync::mpsc::channel();
        let mut messages = Vec::new();
        crawl_candlestick($exchange, $market_type, None, tx, Some(0));
        for msg in rx {
            messages.push(msg);
        }
        assert!(!messages.is_empty());
        assert_eq!(messages[0].exchange, EXCHANGE_NAME.to_string());
        assert_eq!(messages[0].market_type, $market_type);
        assert_eq!(messages[0].msg_type, MessageType::Candlestick);
    }};
}
